function [B,Bse,s2,s2e,lnlike,AIC,H,cH,exitflag]=bipartitePLMM(Y,X,mspp,nspp,V,s2start,MLflag,annealingflag,options,annealingopts)
  
% calls a PLMM for bipartite phylogenetic analyses
 
% X = independent variable
% Y = dependent variable
% mspp = number of species (rows)
% nspp = number of species (columns)
% V = 3-D matrix containing covariance matrices
% s2start = vector of initial values of s2
% MLflag = 1 for ML estimation, 0 for REML
% annealingflag = 1 for initial estimation with simulated annealing, 0 for fminserach
% options = options for fminsearch
% annealingopt = options for annealing
 
% Returns
% B = coefficients from PLMM
% Bse = standard errors of B
% s2 = parameter(s) of the covariance matrix
% s2e = total variance scaling term
% LL = log likelihood
% H = residuals
% cH = Cholesky residuals
% exitflag = 1 if convergence achieved, 0 otherwise

% Requires
% LMM_REML_funct.m
% LMM_ML_funct.m
% anneal.m (from Joachim Vandekerckhove)
    
global tV tX tY count
 
count=1;

[n p]=size(X);
  
% obtain initial values for B from standard regression
B=(X'*X)\(X'*Y);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% perform LMM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tX=X;
tY=Y;
tV=V;

if ~annealingflag
	if ~MLflag
		[s2 LL exitflag]=fminsearch('LMM_REML_funct',s2start,options);
	else
		[s2 LL exitflag]=fminsearch('LMM_ML_funct',s2start,options);
	end
else
	% simulated annealing
	if ~MLflag
		fhandle=@LMM_REML_funct;
	else
		fhandle=@LMM_ML_funct;
	end
	[s2 LL]=anneal(fhandle,s2start,annealingopts);
end
if ~MLflag
	[s2 LL exitflag]=fminsearch('LMM_REML_funct',s2,options);
else
	[s2 LL exitflag]=fminsearch('LMM_ML_funct',s2,options);
end

if ~exitflag
	B=[];
	Bse=[];
	s2=0;
	s2e=0;
	H=[];
	cH=[];
	lnlike=[];
	AIC=[];
	return
end

s2=abs(s2);
C=eye(n);
for i=1:length(s2)
  C=C+s2(i)*tV(:,:,i);
end

invV=C\eye(n);

% compute parameters
B=(tX'*invV*tX)\(tX'*invV*tY);
H=(tY-tX*B);
s2e=(H'*invV*H)/(n-p);

Bs2=s2e*(tX'*invV*tX)^(-1);
Bse=diag(Bs2).^.5;

[n p]=size(tX);
par=p+length(s2);

if MLflag==0
	lnlike=-.5*(n-p)*log(2*pi)+.5*log(det(tX'*tX))-LL;
	AIC=2*par-2*lnlike;
else
	lnlike=-.5*n*log(2*pi)-LL;
	AIC=2*par-2*lnlike;
end


% Cholesky residuals
iD=chol(C)';
D=iD\eye(length(iD));
cH=D*H;

% SVD residuals
% [T L TT]=svd(C);
% D=diag((diag(L).^-.5))*TT';
% cH=D*H;

H=reshape(H,nspp,mspp);
cH=reshape(cH,nspp,mspp);

